#!/bin/perl -w
#******************************************************************************
#                                                                             *
#                                LayGO (tm)                                   *
#           Copyright (c) 1998-2000 by Advanced Relay Corporation             *
#                           All rights reserved.                              *
#                                                                             *
#******************************************************************************
#
#  $Source: q:/laygorcs/pltdrv.pl $
#
#      $Id: pltdrv.pl 1.1 2000/02/29 17:51:47 tiktin Release $
#
#  MODULE DESCRIPTION
#
#	Tk interface for LayGO for Perl5 test program.  This module
#	contains no LayGO code.  All LayGO processing is done in
#	pltdrvsv.pl which executes in another process.  IPC is done
#	using sockets.  The read handle is *not* registered with Tk
#	using fileevent() to automatically process responses from the
#	server because this does not work on all platforms.  Instead,
#	the socket is polled via repeat() using the IO::Select module.
#
#******************************************************************************

use strict;
use Tk;
use Tk::LabFrame;
use Tk::DialogBox;
use Tk::FileSelect;
use Tk::ROText;
use FileHandle;
use IO::Socket;
use IO::Select;
use Time::localtime;
use Config;

require 'images.pl';

my $LOGMODE		= 0;
my $LINENUMBER	= 0;
my $VTX			= 0;
my $FRAMESIZE	= 0;
my $FULLDUPLEX	= 0;
my %CONTROL		= ();
my %STATS		= ('Tx Frames', 0, 'Rx Frames', 0, 'Tx Bytes', 0, 'Rx Bytes', 0);
my $CHILD		= 0;
my $TIMER		= undef;
my $TIME		= 0;
my $START		= 0;
my $DATA		= 0;
my $OPEN		= 0;
my $DATAFILE	= '';
my @SIGNAL		= ('DTR', 'RTS', 'DCD', 'CTS', 'DSR', 'Flags');
my $FONT		= 'systemfixed';
my $MAIN;

my ($READER, $WRITER, $SELECT);
local (*SERVER);

#******************************************************************************
#                                    Main Line                                *
#******************************************************************************

sub MainLine
{
	if (&InitializeServer())
	{
		&InitializeTk();
	}
	else
	{
		print STDERR "pltdrv: Failure initializing server.\n";
	}
}

&MainLine();

#******************************************************************************
#                                   Subroutines                               *
#******************************************************************************

sub InitializeServer
{
	my $host = 'localhost';
	my $port = 2345;

	#
	#	Exec the server with IPC via sockets.
	#
	if ($CHILD = open(SERVER, "| perl pltdrvsv.pl $port"))
	{
		# Wait for server to listen
		sleep(1);

		$READER = IO::Socket::INET->new(PeerAddr => $host,
										PeerPort => $port,
										Proto	 => 'tcp');
		if ($READER)
		{
			$READER->setvbuf(undef, _IONBF, 0);
			$WRITER	= $READER;
			$SELECT = IO::Select->new();
			$SELECT->add($READER);

			return 1;
		}
	}

	return 0;
}

sub RegisterEventHandler
{
	$MAIN->repeat(50, sub { while ($SELECT->can_read(0)) { &PostUpdate(); } });
}

sub SendRequest
{
	$WRITER->send(shift);
}

sub GetMessages
{
	my $s;

	if ($READER->recv($s, 8192))
	{
		chop($s);

		return split("\n", $s);
	}

	return ();
}

sub InitializeTk
{
	$Tk::strictMotif = 1;

	if ($Config{'osname'} =~ /Win/)
	{
		$FONT = 'ansifixed'
	}

	$MAIN = MainWindow->new();
	$MAIN->resizable(0,0);
	$MAIN->protocol('WM_DELETE_WINDOW', \&opExit);

	&InitializeImages();
	&InitializeMainMenu();
	&InitializeMainDisplay();
	&CloseNotify();

	# Center main window
	$MAIN->withdraw();
	$MAIN->update();
	my $gm = sprintf("\+%d\+%d",
				($MAIN->screenwidth()  - $MAIN->reqwidth())  / 2,
				($MAIN->screenheight() - $MAIN->reqheight()) / 2);
	$MAIN->geometry($gm);
	$MAIN->update();

	$MAIN->iconify();
	$MAIN->update();
	$MAIN->deiconify();
	$MAIN->after(100, \&InitializeStack);

	Tk::MainLoop();
}

sub InitializeImages
{
	my %images = (
		'dark',	'leddark',
		'gray',	'ledgray',
		'on',	'ledgreen',
		'off',	'ledred',
		'logo',	'pltdrvlogo'
	);

	foreach my $i (keys(%images))
	{
		$MAIN->Photo($i, -data => eval $images{$i} );
	}
}

sub InitializeStack
{
	foreach my $file (@ARGV)
	{
		&SendRequest("ConfigureStack $file\n");

		if (!&PostUpdate())
		{
			&opExit();
		}
	}

	&SendRequest("InitializeStack\n");

	if (!&PostUpdate())
	{
		&opExit();
	}

	&RegisterEventHandler();
}

sub InitializeMainDisplay
{
	my $frame = $MAIN->Frame()->pack(-fill => 'x');
	my ($f, $w, $s);

	$s = 'Line Number';
	$f = $frame->LabFrame(-label => $s, -labelside => 'acrosstop');
	$f->form(-top => '%0', -left => '%0', -right => '%21', -bottom => '%100');
	$w = $f->Optionmenu(-variable => \$LINENUMBER);
	$w->pack(-fill => 'x', -padx => 10);

	$CONTROL{$s} = $w;

	for (my $i = 0; $i < 32; $i++)
	{
		$w->addOptions(["$i" => $i]);
	}

	$s = 'Start Time';
	$f = $frame->LabFrame(-label => $s, -labelside => 'acrosstop');
	$f->form(-top => '%0', -left => '%21', -right => '%42', -bottom => '%100');
	$w = $f->Label(-text => '00/00/00 00:00:00');
	$w->form(-top => '%0', -left => '%0', -right => '%100', -bottom => '%100');

	$CONTROL{$s} = $w;

	$s = 'Elapsed Time';
	$f = $frame->LabFrame(-label => $s, -labelside => 'acrosstop');
	$f->form(-top => '%0', -left => '%42', -right => '%63', -bottom => '%100');
	$w = $f->Label(-text => '00:00:00:00');
	$w->form(-top => '%0', -left => '%0', -right => '%100', -bottom => '%100');

	$CONTROL{$s} = $w;

	$s = 'State';
	$f = $frame->LabFrame(-label => $s, -labelside => 'acrosstop');
	$f->form(-top => '%0', -left => '%63', -right => '%100', -bottom => '%100');
	$w = $f->Label(-text => 'Closed', -width => 26, -anchor => 'center');
	$w->form(-top => '%0', -left => '%0', -right => '%100', -bottom => '%100');

	$CONTROL{$s} = $w;

	my $subframe = $MAIN->LabFrame(-label => 'Signals', -labelside => 'acrosstop', -borderwidth => 2);
	$subframe->pack(-fill => 'x');

	my $i = 0;

	$w = $subframe->Label(-image => 'dark');
	$w->form(-top => '%0', -left => '%0', -right => '%14');
	$CONTROL{$SIGNAL[$i++]} = $w;

	$w = $subframe->Label(-image => 'dark');
	$w->form(-top => '%0', -left => '%16', -right => '%31');
	$CONTROL{$SIGNAL[$i++]} = $w;

	$w = $subframe->Label(-image => 'dark');
	$w->form(-top => '%0', -left => '%33', -right => '%48');
	$CONTROL{$SIGNAL[$i++]} = $w;

	$w = $subframe->Label(-image => 'dark');
	$w->form(-top => '%0', -left => '%50', -right => '%65');
	$CONTROL{$SIGNAL[$i++]} = $w;

	$w = $subframe->Label(-image => 'dark');
	$w->form(-top => '%0', -left => '%67', -right => '%82');
	$CONTROL{$SIGNAL[$i++]} = $w;

	$w = $subframe->Label(-image => 'dark');
	$w->form(-top => '%0', -left => '%84', -right => '%100');
	$CONTROL{$SIGNAL[$i++]} = $w;

	$i = 0;
	$w = $subframe->Label(-text => $SIGNAL[$i++]);
	$w->form(-top => '%60', -left => '%0', -right => '%14');
	$w = $subframe->Label(-text => $SIGNAL[$i++]);
	$w->form(-top => '%60', -left => '%16', -right => '%31');
	$w = $subframe->Label(-text => $SIGNAL[$i++]);
	$w->form(-top => '%60', -left => '%33', -right => '%48');
	$w = $subframe->Label(-text => $SIGNAL[$i++]);
	$w->form(-top => '%60', -left => '%50', -right => '%65');
	$w = $subframe->Label(-text => $SIGNAL[$i++]);
	$w->form(-top => '%60', -left => '%67', -right => '%82');
	$w = $subframe->Label(-text => $SIGNAL[$i++]);
	$w->form(-top => '%60', -left => '%84', -right => '%100');

	$frame = $MAIN->Frame();

	$subframe = $frame->Frame();

	$f = $subframe->LabFrame(-label => 'Send', -labelside => 'acrosstop', -borderwidth => 2);
	$f->form(-top => '%0', -left => '%0', -right => '%50', -bottom => '%55');
	$w = $f->Label(-text => 'Frames:', -justify => 'right');
	$w->form(-top => '%0', -left => '%0', -right => '%50', -bottom => '%50');
	$w = $f->Label(-text => '0', -justify => 'right');
	$w->form(-top => '%0', -left => '%50', -right => '%100', -bottom => '%50');
	$CONTROL{'Tx Frames'} = $w;
	$w = $f->Label(-text => 'Bytes:', -justify => 'right');
	$w->form(-top => '%50', -left => '%0', -right => '%50', -bottom => '%100');
	$w = $f->Label(-text => '0', -justify => 'right');
	$w->form(-top => '%50', -left => '%50', -right => '%100', -bottom => '%100');
	$CONTROL{'Tx Bytes'} = $w;

	$f = $subframe->LabFrame(-label => 'Receive', -labelside => 'acrosstop', -borderwidth => 2);
	$f->form(-top => '%0', -left => '%50', -right => '%100', -bottom => '%55');
	$w = $f->Label(-text => 'Frames:', -justify => 'right');
	$w->form(-top => '%0', -left => '%0', -right => '%50', -bottom => '%50');
	$w = $f->Label(-text => '0', -justify => 'right');
	$CONTROL{'Rx Frames'} = $w;
	$w->form(-top => '%0', -left => '%50', -right => '%100', -bottom => '%50');
	$w = $f->Label(-text => 'Bytes:', -justify => 'right');
	$w->form(-top => '%50', -left => '%0', -right => '%50', -bottom => '%100');
	$w = $f->Label(-text => '0', -justify => 'right');
	$w->form(-top => '%50', -left => '%50', -right => '%100', -bottom => '%100');
	$CONTROL{'Rx Bytes'} = $w;

	$s = 'Frame Size';
	$f = $subframe->LabFrame(-label => $s, -labelside => 'acrosstop', -borderwidth => 2);
	$f->form(-top => '%55', -left => '%0', -right => '%50', -bottom => '%100');
	$w = $f->Optionmenu(-variable => \$FRAMESIZE);
	$w->pack(-fill => 'x', -padx => 10);

	$CONTROL{$s} = $w;

	$s = 'Vtx';
	$f = $subframe->LabFrame(-label => $s, -labelside => 'acrosstop', -borderwidth => 2);
	$f->form(-top => '%55', -left => '%50', -right => '%100', -bottom => '%100');
	$w = $f->Checkbutton(-text => 'Use verified transmission', -variable => \$VTX, -command => \&opVtx)->pack(-fill => 'x');
	$w->invoke();

	$CONTROL{$s} = $w;

	$subframe->pack(-fill => 'x');

	$s = 'Session Log';
	my $logframe = $frame->LabFrame(-label => $s, -labelside => 'acrosstop', -borderwidth => 2);
	$w = $logframe->Scrolled('ROText', -relief => 'sunken', -borderwidth => '2', -setgrid => 'true',
				   -height => '14', -width => '70', -scrollbars => 'se', -font => $FONT, -wrap => 'none')->pack(-expand => 'yes', -fill => 'both');
	$logframe->pack(-fill => 'x');

	$CONTROL{$s} = $w;

	my $opframe = $frame->LabFrame(-label => 'Operations', -labelside => 'acrosstop', -borderwidth => 2);
	$opframe->pack(-fill => 'x');
	&InitializeOperations($opframe);

	$opframe->form(-top => '%0', -right => '%100', -bottom => '%100');
	$subframe->form(-top => '%0', -left => '%0', -right => $opframe);
	$logframe->form(-top => $subframe, -left => '%0', -right => $opframe, -bottom => '%100');

	$frame->pack(-fill => 'x');
}

sub InitializeMainMenu
{
	my $menu = $MAIN->Frame(-relief => 'raised', -borderwidth => 2);
	$menu->pack(-fill => 'x');

	my $file = $menu->Menubutton(-text => 'File', -underline => 0, -tearoff => 0)->pack();
	$file->command(-label => 'Exit', -underline => 1, -command => \&opExit);

	my $logging	= $menu->Menubutton(-text => 'Logging', -underline => 0, -tearoff => 0)->pack();
	my $i = 0;
	foreach my $label (('None', 'Minimal', 'Normal', 'Verbose'))
	{
		$logging->radiobutton(-label => $label, -variable => \$LOGMODE, -command => \&opLogMode, -value => $i++);
	}
	$logging->cget(-menu)->invoke(2);
	$logging->separator;
	$logging->command(-label => 'Clear', -command => \&opClearLog);

	my $statistics = $menu->Menubutton(-text => 'Statistics', -underline => 0, -tearoff => 0)->pack();
	$CONTROL{'Statistics'} = $statistics->command(-label => 'Statistics...', -command => \&opStatistics);
	$CONTROL{'Status'} = $statistics->command(-label => 'Status...', -command => \&opStatus);
	$statistics->separator;
	$CONTROL{'Clear Statistics'} = $statistics->command(-label => 'Clear Statistics', -command => \&opClearStatistics);
	$statistics->separator;
	$statistics->command(-label => 'Clear Display', -command => \&opClearDisplay);

	$statistics->cget(-menu)->configure(-postcommand => \&opStatisticsPost);

	my $data = $menu->Menubutton(-text => 'Data', -underline => 0, -tearoff => 0)->pack();
	$CONTROL{'Load Data'} = $data->command(-label => 'Load Test Data...', -command => \&opLoadData);
	$data->separator;
	$CONTROL{'Send Data'} = $data->command(-label => 'Send Test Data...', -command => \&opListData);

	$data->cget(-menu)->configure(-postcommand => \&opDataPost);

	my $help = $menu->Menubutton(-text => 'Help', -underline => 0, -tearoff => 0)->pack();
	$help->command(-label => 'Using pltdrv...', -underline => 0, -command => \&opUsing);
	$help->separator;
	$help->command(-label => 'About pltdrv...', -underline => 0, -command => \&opAbout);

	my @pl = (-side => 'left');
	$file->pack(@pl);
	$logging->pack(@pl);
	$statistics->pack(@pl);
	$data->pack(@pl);
	$help->pack(-side => 'right');
}

sub InitializeOperations
{
	my $frame	= shift;
	my @text	= (
		'Open',
		'Configure',
		'Connect',
		'Disconnect',
		'Listen',
		'Listen Cancel',
		'Send',
		'Full Duplex',
		'Reopen',
		'Close'
	);
	my @command  = (
		\&opOpen,
		\&opConfigure,
		\&opConnect,
		\&opDisconnect,
		\&opListen,
		\&opListenCancel,
		\&opSend,
		\&opFullDuplex,
		\&opReopen,
		\&opClose
	);

	for (my $i = 0; $i <= $#text; $i++)
	{
		$CONTROL{$text[$i]} = $frame->Button(-text => $text[$i], -command => $command[$i]);
		$CONTROL{$text[$i]}->pack(-fill => 'x', -padx => 5, -ipadx => 20, -pady => 5);
	}
}

#******************************************************************************
#                                   Operations                                *
#******************************************************************************

sub opExit
{
	&SendRequest("UninitializeStack\n");
	waitpid($CHILD, 0);
	$READER = undef;
	$WRITER = undef;
	close SERVER;
	$MAIN->destroy();
}

sub opOpen
{
	&SendRequest("Open $LINENUMBER\n");
	&opClearLog();

	if (&PostUpdate())
	{
		&OpenNotify();
		&PostAllSignals('gray');
	}
}

sub opConfigure
{
	my $fileSelect	= $MAIN->FileSelect(-directory => '.');
	my $file		= $fileSelect->Show();

	if (defined $file && $file ne '')
	{
		&SendRequest("Configure $file\n");
	}
}

sub opConnect
{
	&SendRequest("Connect\n");

	&Start();
}

sub opLogMode
{
	&SendRequest("LogMode $LOGMODE\n");
}

sub opDisconnect
{
	&SendRequest("Disconnect\n");

	$CONTROL{'Disconnect'}->configure(-state => 'disabled');
}

sub opListen
{
	&SendRequest("Listen\n");

	if (&Start())
	{
		$CONTROL{'Listen Cancel'}->configure(-state => 'normal');
	}
}

sub opListenCancel
{
	&SendRequest("ListenCancel\n");

	$CONTROL{'Listen Cancel'}->configure(-state => 'disabled');
}

sub opSend
{
	&SendRequest("Send $FRAMESIZE\n");
}

sub opVtx
{
	&SendRequest("Vtx $VTX\n");
}

sub opFullDuplex
{
	$FULLDUPLEX = !$FULLDUPLEX;

	if ($FULLDUPLEX)
	{
		$CONTROL{'Disconnect'}->configure(-state => 'disabled');
		$CONTROL{'Send'}->configure(-state => 'disabled');
		$CONTROL{'Reopen'}->configure(-state => 'disabled');
		$CONTROL{'Full Duplex'}->configure(-state => 'active');
	}
	else
	{
		$CONTROL{'Disconnect'}->configure(-state => 'normal');
		$CONTROL{'Send'}->configure(-state => 'normal');
		$CONTROL{'Reopen'}->configure(-state => 'normal');
		$CONTROL{'Full Duplex'}->configure(-state => 'normal');
	}

	&SendRequest("FullDuplex $FRAMESIZE\n");
}

sub opReopen
{
	&SendRequest("Reopen\n");

	$CONTROL{'Reopen'}->configure(-state => 'disabled');
}

sub opClose
{
	&SendRequest("Close\n");

	if (&PostUpdate)
	{
		&CloseNotify();
	}
}

sub opClearLog
{
    $CONTROL{'Session Log'}->delete('1.0', 'end');
}

sub opStatistics
{
	&SendRequest("Statistics\n");
}

sub opClearStatistics
{
	&SendRequest("ClearStatistics\n");
}

sub opStatus
{
	&SendRequest("Status\n");
}

sub opClearDisplay
{
	foreach my $key (keys(%STATS))
	{
		$STATS{$key} = 0;
	    $CONTROL{$key}->configure(-text => '0');
	}
}

sub opLoadData
{
	my $load = 1;

	if ($DATA > 0)
	{
		my $question =
			"Test data frames are currently loaded from\n$DATAFILE.\n" .
			"These frames will be discarded if you load\n" .
			"frames from another file.\n\n" .
			"Are you sure you want to load frames from\n" .
			"another file?";
		my $response = $MAIN->messageBox(-icon => 'questhead',
								-message => $question, -title => 'Question',
								-type => 'YesNoCancel', -default => 'Cancel');

		$load = $response eq 'Yes';
	}

	if ($load)
	{
		my $fileSelect	= $MAIN->FileSelect(-directory => '.');
		my $file		= $fileSelect->Show();

		if (defined $file && $file ne '')
		{
			$DATAFILE = $file;
			&SendRequest("LoadData $file\n");
		}
	}
}

sub opListData
{
	&SendRequest("ListData\n");
}

sub opSendData
{
	my ($i) = shift()->curselection();

	&SendRequest("SendData $i\n");
}

sub opViewData
{
	my ($i) = shift()->curselection();

	&SendRequest("ViewData $i\n");
}

sub opUsing
{
	&PostUsingDialog();
}

sub opAbout
{
	&PostAboutDialog();
}

sub opStatisticsPost
{
	my $state = $OPEN ? 'normal' : 'disabled';

	$CONTROL{'Statistics'}->configure(-state => $state);
	$CONTROL{'Status'}->configure(-state => $state);
	$CONTROL{'Clear Statistics'}->configure(-state => $state);
}

sub opDataPost
{
	my $state = $DATA > 0 ? $CONTROL{'Send'}->cget('state') : 'disabled';

	$CONTROL{'Send Data'}->configure(-state => $state);
}

#******************************************************************************
#                             Support Routines                                *
#******************************************************************************

sub Start
{
	if (&PostUpdate())
	{
		&opClearDisplay();
		&PostAllSignals('gray');
		$CONTROL{'Configure'}->configure(-state => 'disabled');
		$CONTROL{'Connect'}->configure(-state => 'disabled');
		$CONTROL{'Listen'}->configure(-state => 'disabled');
		$CONTROL{'Reopen'}->configure(-state => 'normal');
		$CONTROL{'Close'}->configure(-state => 'disabled');
		$CONTROL{'Elapsed Time'}->configure(-text => '00:00:00:00');

		my $s = sprintf("%02d/%02d/%02d %02d:%02d:%02d",
							localtime->mon()+1,
							localtime->mday(),
							localtime->year(),
							localtime->hour(),
							localtime->min(),
							localtime->sec());
		$CONTROL{'Start Time'}->configure(-text => $s);

		$FULLDUPLEX = 0;
		$START		= time();
		$TIME		= $START;
		$TIMER		= $MAIN->repeat(100, \&Timer);
		return 1;
	}

	return 0;
}

sub Stop
{
	if (defined $TIMER)
	{
		$MAIN->afterCancel($TIMER);
		$TIMER = undef;
	}
}

sub OpenNotify
{
	&Stop();

	$CONTROL{'Open'}->configure(-state => 'disabled');
	$CONTROL{'Configure'}->configure(-state => 'normal');
	$CONTROL{'Connect'}->configure(-state => 'normal');
	$CONTROL{'Disconnect'}->configure(-state => 'disabled');
	$CONTROL{'Listen'}->configure(-state => 'normal');
	$CONTROL{'Listen Cancel'}->configure(-state => 'disabled');
	$CONTROL{'Send'}->configure(-state => 'disabled');
	$CONTROL{'Full Duplex'}->configure(-state => 'disabled');
	$CONTROL{'Reopen'}->configure(-state => 'disabled');
	$CONTROL{'Close'}->configure(-state => 'normal');
	$CONTROL{'Line Number'}->configure(-state => 'disabled');

	$MAIN->title("pltdrv - Line $LINENUMBER");
	$OPEN = 1;
}

sub CloseNotify
{
	$CONTROL{'Open'}->configure(-state => 'normal');
	$CONTROL{'Configure'}->configure(-state => 'disabled');
	$CONTROL{'Connect'}->configure(-state => 'disabled');
	$CONTROL{'Disconnect'}->configure(-state => 'disabled');
	$CONTROL{'Listen'}->configure(-state => 'disabled');
	$CONTROL{'Listen Cancel'}->configure(-state => 'disabled');
	$CONTROL{'Send'}->configure(-state => 'disabled');
	$CONTROL{'Full Duplex'}->configure(-state => 'disabled');
	$CONTROL{'Reopen'}->configure(-state => 'disabled');
	$CONTROL{'Close'}->configure(-state => 'disabled');
	$CONTROL{'Line Number'}->configure(-state => 'normal');

	$CONTROL{'Elapsed Time'}->configure(-text => '00:00:00:00');
	$CONTROL{'Start Time'}->configure(-text => '00/00/00 00:00:00');

	&opClearDisplay();
	&opClearLog();

	&PostAllSignals('dark');
	$MAIN->title('pltdrv - Closed');
	$OPEN = 0;
}

sub ConnectNotify
{
	$CONTROL{'Listen Cancel'}->configure(-state => 'disabled');
	$CONTROL{'Disconnect'}->configure(-state => 'normal');
	$CONTROL{'Send'}->configure(-state => 'normal');
	$CONTROL{'Full Duplex'}->configure(-state => 'normal');
}

sub Timer
{
	my $now	= time();

	if ($now != $TIME)
	{
		my $seconds	= $now - $START;
		my $days	= $seconds / 86400; $seconds = $seconds % 86400;
		my $hours	= $seconds /  3600; $seconds = $seconds %  3600;
		my $minutes	= $seconds /    60; $seconds = $seconds %    60;

		my $s = sprintf("%02d:%02d:%02d:%02d", $days, $hours, $minutes, $seconds);

		$CONTROL{'Elapsed Time'}->configure(-text => $s);

		$TIME = $now;
	}
}

sub PostErrorDialog
{
	$MAIN->messageBox(-icon => 'warning', -message => shift, -title => 'Error', -type => 'OK');
}

sub PostFatalDialog
{
	my $message = shift;

	$message =~ tr/\t/\n/;

	$MAIN->messageBox(-icon => 'error', -message => $message, -title => 'Fatal Error', -type => 'OK');
}

sub PostInformationDialog
{
	$MAIN->messageBox(-icon => 'info', -message => shift, -title => 'Information', -type => 'OK');
}

sub PostTextDialog
{
	my ($title, $height, $width, $text) = @_;
	my $dialog	= $MAIN->DialogBox(-title => $title, -buttons => ['OK']);
	my $widget	= $dialog->Scrolled('ROText', -relief => 'sunken', -borderwidth => 2, -setgrid => 'true',
				   -height => $height, -width => $width, -wrap => 'none', -scrollbars => 'se', -font => $FONT);
	$widget->pack(-expand => 'yes', -fill => 'both', -padx => 10, -pady => 10);
	$widget->insert('end', $text);
	$dialog->Show();
}

sub CenterDialogOver
{
	my ($w, $p) = @_;

	$w->withdraw();
	$w->update();
	my $gm = sprintf("\+%d\+%d",
				(($p->width()  - $w->reqwidth())  / 2) + $p->x(),
				(($p->height() - $w->reqheight()) / 2) + $p->y());
	$w->geometry($gm);
	$w->update();

	return $gm;
}

sub PostUpdate
{
	my @msglist	= &GetMessages();
	my $success	= 1;

	for(my $i = 0; $i <= $#msglist; $i++)
	{
		my $s = $msglist[$i];

		if ($s =~ m/^Failure/)
		{
			&PostErrorDialog($s);
			$success = 0;
		}
		if ($s =~ m/^Fatal/)
		{
			&PostFatalDialog($s);
			$success = 0;
		}
		elsif ($s =~ m/^FrameStat/)
		{
			&PostFrameStat($s);
		}
		elsif ($s =~ m/^Configure/)
		{
			&PostConfigure($s);
		}
		elsif ($s =~ m/^ListData/)
		{
			&PostListData($s);
		}
		elsif ($s =~ m/^LoadData/)
		{
			&PostLoadData($s);
		}
		elsif ($s =~ m/^MaxBuffer/)
		{
			&PostMaxBuffer($s);
		}
		elsif ($s =~ m/^ViewData/)
		{
			&PostViewData($s);
		}
		elsif ($s =~ m/^Signal/)
		{
			&PostSignal($s);
		}
		elsif ($s =~ m/^State/)
		{
			&PostState($s);
		}
		elsif ($s =~ m/^Status/)
		{
			&PostStatus($s);
		}
		elsif ($s =~ m/^Statistics/)
		{
			&PostStatistics($s);
		}
		else
		{
			&PostLogEntry($s);
		}
	}

	return $success;
}

sub PostConfigure
{
	my $s = shift;

	$s =~ m/^Configure (.*)$/;

	PostInformationDialog($1);
}

sub PostFrameStat
{
	my @args = split(/\s/, shift());
	my $f;
	my $b;

	if ($#args > 1)
	{
		if ($args[1] =~ m/Rx/)
		{
			$f = 'Rx Frames';
			$b = 'Rx Bytes';
		}
		else
		{
			$f = 'Tx Frames';
			$b = 'Tx Bytes';
		}

		$STATS{$f}++;
		$STATS{$b} += $args[2];
		$CONTROL{$f}->configure(-text => $STATS{$f});
		$CONTROL{$b}->configure(-text => $STATS{$b});
	}
}

sub PostState
{
	my $s = shift;

	$s =~ m/^State (.*)$/;

	$CONTROL{'State'}->configure(-text => $1);

	if ($1 =~ m/^Data/)
	{
		&ConnectNotify();
	}
	elsif ($1 =~ m/^Open/)
	{
		&OpenNotify();
	}
}

sub PostStatus
{
	my $s = shift;

	$s =~ tr/\t/\n/;

	PostTextDialog('Status Report', 15, 45, $s);
}

sub PostStatistics
{
	my $s = shift;

	$s =~ tr/\t/\n/;

	PostTextDialog('Statistics', 15, 45, $s);
}

sub PostSignal
{
	my @args = split(/\s/, shift());

	if ($#args > 1)
	{
		my $w = $CONTROL{$args[1]};

		if (defined $w)
		{
			$w->configure(-image => $args[2]);
		}
	}
}

sub PostListData
{
	my @args = split(/\t/, shift());

	if ($#args > 1)
	{
		my $tdat	= $MAIN->Toplevel(-title => 'Send Test Data');
		my $widget	= $tdat->Scrolled('Listbox', -relief => 'sunken', -borderwidth => '2',
					   -selectmode => 'single', -height => '15', -width => '45', -scrollbars => 'se')->pack(-expand => 'yes', -fill => 'both');

		for (my $i = 1; $i <= $#args; $i++)
		{
			$widget->insert('end', $args[$i]);
		}

		$widget->selectionSet(0);

		my $f = $tdat->Frame()->pack(-fill => 'x');
		my $i;
		$i = $f->Button(-text => 'Send', -command => sub { &opSendData($widget); })->pack();
		$i->form(-top => '%5', -left => '%7', -right => '%31', -bottom => '%95');
		$i = $f->Button(-text => 'View...', -command => sub { &opViewData($widget); })->pack();
		$i->form(-top => '%5', -left => '%38', -right => '%62', -bottom => '%95');
		$i = $f->Button(-text => 'Done', -command => sub { $tdat->destroy; })->pack();
		$i->form(-top => '%5', -left => '%69', -right => '%93', -bottom => '%95');
	}
}

sub PostLoadData
{
	my $s = shift;

	$s =~ m/^LoadData (.*)$/;

	PostInformationDialog($1);

	$1 =~ m/^([0-9]+)/;

	$DATA = $1;
}

sub PostViewData
{
	my @args = split(/\t/, shift());

	if ($#args > 1)
	{
		my $view	= $MAIN->DialogBox(-title => 'Test Data', -buttons => ['OK']);
		my $s		= "Contents of test data frame:\n\n${args[1]}\n\n";

		for (my $i = 2; $i <= $#args; $i++)
		{
			$s = $s . $args[$i] . "\n";
		}

		$view->add('Label', -justify => 'left', -text => $s, -font => $FONT)->pack();
		$view->Show();
	}
}

sub PostMaxBuffer
{
	my ($max)	= shift() =~ m/^MaxBuffer (.*)$/;
	my $w		= $CONTROL{'Frame Size'};
	my $n		= 0;

	for (my $i = 2; $i < $max; $i <<= 1)
	{
		$w->addOptions(["$i" => $i]);
		$n++;
	}

	$w->addOptions(["$max" => $max]);
	$n++;

	$w->cget(-menu)->invoke(6 < $n ? 6 : $n - 1);
}

sub PostLogEntry
{
	if ($LOGMODE > 0)
	{
		my $log = $CONTROL{'Session Log'};

		if ($log->index('end') > 1000)
		{
		    $log->delete('1.0', '500.0');
		}

		$log->insert('end', shift() . "\n");
		$log->see('end');
	}
}

sub PostAboutDialog
{
	my $about = $MAIN->DialogBox(-title => 'About pltdrv', -buttons => ['OK']);
	$about->add('Label', -image => 'logo')->pack(-side => 'left', -padx => 7);
	$about->add('Label', -justify => 'left', -text => "pltdrv\nVersion 2.20\nCopyright (c) 1998-2000\nAdvanced Relay Corporation\nAll rights reserved")->pack(-side => 'left');
	$about->Show();
}

sub PostUsingDialog
{
	PostTextDialog('Using pltdrv', 25, 62, &usingpltdrv());
}

sub PostAllSignals
{
	my $s = shift;

	for (my $i = 0; $i <= $#SIGNAL ; $i++)
	{
		$CONTROL{$SIGNAL[$i]}->configure(-image => $s);
	}
}

1;

#******************************************************************************
#                                   End of File                               *
#******************************************************************************
