#!/bin/perl -w
#******************************************************************************
#                                                                             *
#                                LayGO (tm)                                   *
#           Copyright (c) 1998-2000 by Advanced Relay Corporation             *
#                           All rights reserved.                              *
#                                                                             *
#******************************************************************************
#
#  $Source: q:/laygorcs/pltdrvsv.pl $
#
#      $Id: pltdrvsv.pl 1.1 2000/02/29 17:51:48 tiktin Release $
#
#  MODULE DESCRIPTION
#
#    LayGO processing server.
#
#******************************************************************************

use strict;
use Laygo;
use Laygo::X21bis;
use IO::Socket;
use IO::Select;
use FileHandle;

require 'hex.pl';
require 'status.pl';
require 'statistics.pl';
require 'tdat.pl';
require 'vtx.pl';

my $LOGMODE		= 0;
my $VTX			= 0;
my $SIZE		= 0;
my $FULLDUPLEX	= 0;
my $SEND		= 0;
my $INITIALIZED	= 0;
my @DATA		= ();
my $CID			= Laygo::INVALID_CID;
my $STATE		= Laygo::STATE_CLOSED;
my $SIGNALS		= 0;
my $SIGNALCFG	= 0;
my %SIGNAL 		= (
	phys_SIGNAL_CTS, 'CTS',
	phys_SIGNAL_DCD, 'DCD',
	phys_SIGNAL_DSR, 'DSR',
	phys_SIGNAL_FLG, 'Flags',
	phys_SIGNAL_RTS, 'RTS',
	phys_SIGNAL_DTR, 'DTR',
);
my $SELECT		= IO::Select->new();
my %JUMPTABLE	= (
	'Open',					\&opOpen,
	'Configure',			\&opConfigure,
	'Connect',				\&opConnect,
	'Disconnect',			\&opDisconnect,
	'Listen',				\&opListen,
	'ListenCancel',			\&opListenCancel,
	'Send',					\&opSend,
	'FullDuplex',			\&opFullDuplex,
	'Reopen',				\&opReopen,
	'Close',				\&opClose,
	'Statistics',			\&opStatistics,
	'ClearStatistics',		\&opClearStatistics,
	'Status',				\&opStatus,
	'LoadData',				\&opLoadData,
	'SendData',				\&opSendData,
	'ListData',				\&opListData,
	'ViewData',				\&opViewData,
	'LogMode',				\&opLogMode,
	'Vtx',					\&opVtx,
	'ConfigureStack',		\&opConfigureStack,
	'InitializeStack',		\&opInitializeStack,
	'UninitializeStack',	\&opUninitializeStack,
);
my $MINBUFFER	= 1024;
my $MAXBUFFER	= 0;
my $INBUFFER;
my $OUTBUFFER;

my ($READER, $WRITER);

#******************************************************************************
#                                    Main Line                                *
#******************************************************************************

sub MainLine
{
	if (&InitializeServer())
	{
		&MainLoop();
	}
	else
	{
		print STDERR "pltdrvsv: Failure initializing.\n";
	}
}

&MainLine();

#******************************************************************************
#                                   Subroutines                               *
#******************************************************************************

sub InitializeServer
{
	my $port	= $ARGV[0];
	my $server	= IO::Socket::INET->new(Listen    => 1,
										LocalPort => $port,
										Proto     => 'tcp',
										Reuse     => 'true',
										Timeout   => 5.0);

	if ($server)
	{
		$READER = $server->accept();
		$server = undef;

		if (defined $READER)
		{
			$READER->setvbuf(undef, _IONBF, 0);
			$SELECT->add($READER);
			$WRITER	= $READER;

			return 1;
		}
		else
		{
			print STDERR "pltdrvsv: Listen timeout.\n";
		}
	}
	else
	{
		print STDERR "pltdrvsv: Failure listening on port $port.\n";
	}

	return 0;
}

sub SendMessage
{
	my $s = shift;

	$WRITER->send($s);
}

sub GetRequests
{
	if ($SELECT->can_read(0))
	{
		my $s;

		if ($READER->recv($s, 8192))
		{
			chop($s);
			print STDERR "got: $s.\n";

			return split("\n", $s);
		}
	}

	return ();
}

sub MainLoop
{
	while (1)
	{
		my @reqlist;
		my $i;

		if (@reqlist = &GetRequests())
		{
			for($i = 0; $i <= $#reqlist; $i++)
			{
				my @args = split(/\s/, $reqlist[$i]);
				my $op;

				if (defined ($op = $JUMPTABLE{$args[0]}))
				{
					shift(@args);

					&$op(@args);
				}
				else
				{
					print STDERR "Internal Error: Undefined operation $args[0].\n";
				}
			}
		}
		else
		{
			my $waitTime = 0.1;

			if ($STATE == Laygo::STATE_WAITING_FOR_REMOTE_ACCEPT	||
				$STATE == Laygo::STATE_LISTENING					||
				$STATE == Laygo::STATE_RESETTING					||
				$STATE == Laygo::STATE_WAITING_FOR_REMOTE_CONFIRMATION)
			{
				&ReadEvent();
			}
			elsif ($STATE == Laygo::STATE_DATA_TRANSFER_XON)
			{
				if ($FULLDUPLEX)
				{
					&FullDuplex();
				}
				else
				{
					if ($SEND > 0)
					{
						if (&SendFrame())
						{
							$SEND--;
						}
					}
					&ReceiveFrame();
				}
			}
			elsif ($STATE == Laygo::STATE_DATA_TRANSFER_XOFF)
			{
				&ReceiveFrame();
			}
			elsif ($STATE == Laygo::STATE_OPEN || $STATE == Laygo::STATE_CLOSED)
			{
				# do nothing
			}
			else
			{
				print STDERR "Internal Error: Undefined state $STATE.\n";
			}

			if ($waitTime > 0)
			{
				select(undef, undef, undef, $waitTime);
			}
		}
	}
}

#******************************************************************************
#                                   Operations                                *
#******************************************************************************

sub opOpen
{
	my $line	= shift;
	my $result	= lgo_OpenProtocol(Laygo::PROTOCOL_X21_BIS, $line, 0);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure opening line $line: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		$CID = $result;

		&SendMessage("Line $line open with CID = $CID\n");
		&UpdateState();
	}
}

sub opConfigure
{
	my $file	= shift;
	my $result	= lgo_LoadConfiguration($file,
						Laygo::PROTOCOL_X21_BIS, $INBUFFER, $MAXBUFFER);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure loading configuration from $file: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		if (($result = lgo_SetConfiguration($CID, $INBUFFER, $result)) < 0)
		{
			&SendMessage(sprintf("Failure setting configuration on CID: %s.\n", lgo_ErrorMessage($result)));
		}
		else
		{
			&SendMessage("CID $CID configured from $file.\n");
		}
	}
}

sub opConnect
{
	my $result = lgo_ConnectRequest($CID, $Laygo::NULL, 0);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure on connect request: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&Start();
	}
}

sub opDisconnect
{
	my $result = lgo_DisconnectRequest($CID, $Laygo::NULL, 0);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure on disconnect request: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&UpdateState();
	}
}

sub opListen
{
	my $result = lgo_Listen($CID);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure on listen request: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		$SEND		= 0;
		@DATA		= ();
		$FULLDUPLEX = 0;
		$SIGNALS	= 0;
		$SIGNALCFG	= 0;
		&vtxReset();
		&UpdateState();
	}
}

sub opListenCancel
{
	my $result = lgo_ListenCancel($CID);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure on listen cancel: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&UpdateState();
	}
}

sub opSend
{
	$SEND++;
	$SIZE = shift;
}

sub opVtx
{
	$VTX = shift;
}

sub opFullDuplex
{
	$FULLDUPLEX = !$FULLDUPLEX;
	$SIZE		= shift;
}

sub opReopen
{
	my $result = lgo_Reopen($CID);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure on reopen request: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&UpdateSignals();
		&UpdateState();
	}
}

sub opClose
{
	my $result = lgo_Close($CID);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure on close request: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		$CID = Laygo::INVALID_CID;
		&UpdateState();
	}
}

sub opStatistics
{
	my $result = lgo_GetStatistics($CID, $INBUFFER, $MAXBUFFER);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure getting statistics: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&SendMessage(&formatStatisticsTable($INBUFFER, Laygo::PROTOCOL_X21_BIS));
	}
}

sub opClearStatistics
{
	my $result	= lgo_ClearStatistics($CID);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure clearing statistics: %s\n", lgo_ErrorMessage($result)));
	}
}

sub opStatus
{
	my $result = lgo_GetStatus($CID, $INBUFFER, $MAXBUFFER);

	if ($result < 0)
	{
		&SendMessage(sprintf("Failure getting status: %s\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&SendMessage(&formatStatusReport($INBUFFER));
	}
}

sub opLoadData
{
	my $file	= shift;
	my $count	= &tdatLoad($file);

	if ($count > 0)
	{
		&SendMessage("LoadData $count test frames loaded from $file.\n");
	}
	else
	{
		&SendMessage("Failure loading test data from '$file'.\n");
	}
}

sub opSendData
{
	my $i = shift;

	if ($i >= 0 && $i < &tdatGetCount())
	{
		push @DATA, $i;
		$SEND++;
	}
}

sub opListData
{
	&SendMessage("ListData" . &tdatListData() . "\n");
}

sub opViewData
{
	my $i = shift;

	if ($i >= 0 && $i < &tdatGetCount())
	{
		my $l = &tdatGetLabel($i);
		my $s = "ViewData\t$l\t";

		$s .= &hexFormat(&tdatGetFrame($i), &tdatGetSize($i), 16);

		$s =~ tr/\n/\t/;

		&SendMessage($s . "\n");
	}
}

sub opLogMode
{
	$LOGMODE = shift;
}

sub opConfigureStack
{
	my $file	= shift;
	my $result	= lgo_ConfigureStack($file);

	if ($result < 0)
	{
		&SendMessage(sprintf("Fatal Error!\t\tFailure configuring stack with '$file': %s.\n", lgo_ErrorMessage($result)));
	}
	else
	{
		&SendMessage(sprintf("Stack configured with $file.\n"));
	}
}

sub opInitializeStack
{
	my $result = lgo_InitializeStack(1,1);

	if ($result < 0)
	{
		&SendMessage(sprintf("Fatal Error!\t\tFailure initializing stack: %s.\n", lgo_ErrorMessage($result)));
	}
	else
	{
		$INITIALIZED = 1;

		lgo_EnableStack();

		&SendMessage("Stack initialized and enabled.\n");

		$MAXBUFFER	= lgo_MaxBufferSize(Laygo::INVALID_CID);

		&SendMessage("MaxBuffer $MAXBUFFER\n");

		$MAXBUFFER	= $MAXBUFFER < $MINBUFFER ? $MINBUFFER : $MAXBUFFER;
		$INBUFFER	= lgo_BufferNew($MAXBUFFER);
		$OUTBUFFER	= lgo_BufferNew($MAXBUFFER);

		hexFill($OUTBUFFER, 0, $MAXBUFFER, 1);

		&vtxInitialize($MAXBUFFER);
	}
}

sub opUninitializeStack
{
	if ($INITIALIZED)
	{
		if ($STATE != Laygo::STATE_CLOSED)
		{
			lgo_Close($CID);
		}

		lgo_DisableStack();
		lgo_UninitializeStack();
	}

	exit(0);
}

#******************************************************************************
#                               Support Routines                              *
#******************************************************************************

sub FullDuplex
{
	my $i = 16;

	while ($i-- && &ReceiveFrame())
	{
		;
	}

	$i = 8;

	while ($i-- && &SendFrame())
	{
		;
	}
}

sub Start
{
	$SEND		= 0;
	@DATA		= ();
	$FULLDUPLEX = 0;
	$SIGNALS	= 0;
	$SIGNALCFG	= 0;
	&vtxReset();
	&UpdateState();
}

sub ReadEvent
{
	my $size	= $MAXBUFFER;
	my $result	= lgo_Event($CID, $INBUFFER, \$size);

	&UpdateSignals();

	if ($result < 0)
	{
		return 0;
	}
	else
	{
		if ($result == Laygo::EVENT_DATA_LENGTH_ERROR)
		{
			LogReceiveFrame(Laygo::EVENT_DATA_LENGTH_ERROR, $size);
		}
		elsif ($result == Laygo::EVENT_DATA_CRC_ERROR)
		{
			LogReceiveFrame(Laygo::EVENT_DATA_CRC_ERROR, $size);
		}
		elsif ($result == Laygo::EVENT_DATA_ABORTED_FRAME)
		{
			LogReceiveFrame(Laygo::EVENT_DATA_ABORTED_FRAME, $size);
		}
		else
		{
			&SendMessage(sprintf("Event received: %s.\n", lgo_EventMessage($result)));
			&UpdateState();
		}
		return 1;
	}
}

sub SendFrame
{
	if ($#DATA > -1)
	{
		return &SendDataFrame();
	}
	else
	{
		my $buffer = $OUTBUFFER;

		if ($VTX)
		{
			$buffer = vtxGetTx($SIZE);
		}

		my $result = lgo_Write($CID, $buffer, $SIZE);

		if ($result > 0)
		{
			&SendMessage("FrameStat Tx $SIZE\n");

			if ($VTX)
			{
				vtxNextTx();
			}

			return 1;
		}
	}

	return 0;
}

sub SendDataFrame
{
	my $i = shift @DATA;
	my $frame;
	my $size;

	if (($frame = &tdatGetFrame($i)) && ($size = tdatGetSize($i)))
	{
		my $result = lgo_Write($CID, $frame, $size);

		if ($result > 0)
		{
			&SendMessage("FrameStat Tx $size\n");

			return 1;
		}
		else
		{
			unshift @DATA, $i;
		}
	}
	else
	{
		$SEND--;
	}

	return 0;
}

sub ReceiveFrame
{
	my $result = lgo_Read($CID, $INBUFFER, $MAXBUFFER);

	if ($result > 0)
	{
		&LogReceiveFrame(Laygo::EVENT_DATA_OK, $result);

		return 1;
	}
	elsif ($result == Laygo::ERROR_EVENT_WAITING)
	{
		&ReadEvent();
	}

	return 0;
}

sub LogReceiveFrame
{
	my ($frame, $result) = @_;

	&SendMessage("FrameStat Rx $result\n");

	if ($LOGMODE > 0)
	{
		my $str = "Unknown";
		my $vtx	= "";

		if ($frame == Laygo::EVENT_DATA_OK)
		{
			$str = "Good";
		}
		elsif ($frame == Laygo::EVENT_DATA_LENGTH_ERROR)
		{
			$str = "Length Error";
		}
		elsif ($frame == Laygo::EVENT_DATA_CRC_ERROR)
		{
			$str = "CRC Error";
		}
		elsif ($frame == Laygo::EVENT_DATA_ABORTED_FRAME)
		{
			$str = "Abort";
		}

		if ($VTX && $frame == Laygo::EVENT_DATA_OK)
		{
			$vtx = vtxValidateRx($INBUFFER, $result);
		}

		&SendMessage(sprintf("> %4d : $str $vtx\n", $result));

		if ($LOGMODE == 2)
		{
			&SendMessage(&hexFormat($INBUFFER, $result > 16 ? 16 : $result, 16));
		}
		elsif ($LOGMODE == 3)
		{
			&SendMessage(&hexFormat($INBUFFER, $result, 16));
		}
	}
}

sub UpdateState
{
	$STATE	= lgo_State($CID);

	&SendMessage(sprintf("State %s\n", lgo_StateMessage($STATE)));
}

sub GetSignals
{
	my $size	= $MAXBUFFER;
	my $result	= lgo_Ioctl($CID, phys_IOCTL_GET_SIGNALS, $Laygo::NULL, 0, $INBUFFER, \$size);

	if ($result >= 0 && $size == phys_SIZEOF_SIGNAL_DATA)
	{
		my $data	=  &hexGetLong($INBUFFER, 0);
		my $sig		= ($data >> 16) & 0xFFFF;
		my $cfg		= $data & 0xFFFF;

		return ($sig, $cfg);
	}

	return (0, 0);
}

sub UpdateSignals
{
	my ($sig, $cfg) = &GetSignals();
	my $change;
	my $key;

	if ($SIGNALCFG != $cfg)
	{
		foreach $key (keys(%SIGNAL))
		{
			&SignalCfgChange($cfg, $key);
		}

		$SIGNALCFG = $cfg;
	}

	$change = ($SIGNALS ^ $sig) & $SIGNALCFG;

	if ($change != 0)
	{
		foreach $key (keys(%SIGNAL))
		{
			&SignalChange($change, $key);
		}
	}

	$SIGNALS = $sig;
}

sub SignalCfgChange
{
	my ($config, $signal) = @_;

	if (($config & $signal) == 0)
	{
		&SendMessage("Signal $SIGNAL{$signal} gray\n");
	}
	elsif (($SIGNALS & $signal) == 0)
	{
		&SendMessage("Signal $SIGNAL{$signal} off\n");
	}
}

sub SignalChange
{
	my ($change, $signal) = @_;

	if (($change & $signal) != 0)
	{
		if (($SIGNALS & $signal) != 0)
		{
			&SendMessage("Signal $SIGNAL{$signal} off\n");
		}
		else
		{
			&SendMessage("Signal $SIGNAL{$signal} on\n");
		}
	}
}

1;

#******************************************************************************
#                                   End of File                               *
#******************************************************************************
