#!/bin/perl -w
#******************************************************************************
#                                                                             *
#                                LayGO (tm)                                   *
#           Copyright (c) 1998-2000 by Advanced Relay Corporation             *
#                           All rights reserved.                              *
#                                                                             *
#******************************************************************************
#
#  $Source: q:/laygorcs/vtx.pl $
#
#      $Id: vtx.pl 1.1 2000/02/29 17:51:49 tiktin Release $
#
#  MODULE DESCRIPTION
#
#    Exports verified transmission frame handling routines.
#
#******************************************************************************

use strict;
use Laygo;

require 'hex.pl';

my $MAX;
my $FORWARD;
my $REVERSE;
my $TXSEQUENCE;
my $TXFORWARD;
my $RXSEQUENCE;

my $MODULUS			= 7;
my $SIZEOF_HEADER	= 2;
my $BUFFER_MIN		= $SIZEOF_HEADER;
my $DIRECTION_BIT	= 1;
my $SEQUENCE_BITS	= 0x0E;
my $SIZE_LOW_BITS	= 0xF0;
my $SIZE_HIGH_BITS	= 0xFF;
my $SIZE_LOW_NIBBLE	= 0x0F;

my $ERROR_INVALID_BUFFER_SIZE			= '[Vtx Error: invalid buffer size]';
my $ERROR_SEQUENCE_NUMBER_DISCREPANCY	= '[Vtx Error: sequence number discrepancy]';
my $ERROR_SIZE_DISCREPANCY				= '[Vtx Error: size discrepancy]';
my $ERROR_DATA_DISCREPANCY				= '[Vtx Error: data discrepancy]';
my $ERROR_NONE							= '[Vtx Good]';

sub vtxInitialize
{
	$MAX = shift();

	$FORWARD = lgo_BufferNew($MAX);
	$REVERSE = lgo_BufferNew($MAX);

	hexFill($FORWARD, 2, $MAX, 1);
	hexFill($REVERSE, 2, $MAX, 0);

	&vtxReset();
}

sub vtxGetTx
{
	my $data = $TXFORWARD ? $FORWARD : $REVERSE;
	my $size = shift();
	my $byte;

	$byte = (((($size - 1) & $SIZE_LOW_NIBBLE) << 4) |
				((($TXSEQUENCE % $MODULUS) << 1) & $SEQUENCE_BITS) |
				($TXFORWARD ? 0 : 1));
	lgo_BufferSet($data, 0, $byte);

	$byte = ((($size - 1) >> 4) & $SIZE_HIGH_BITS);
	lgo_BufferSet($data, 1, $byte);

	return $data;
}

sub vtxNextTx
{
	$TXSEQUENCE++;
	$TXFORWARD = !$TXFORWARD;
}

sub vtxReset
{
	&vtxResetRx();
	&vtxResetTx();
}

sub vtxResetRx()
{
	$RXSEQUENCE	= 0;
}

sub vtxResetTx()
{
	$TXSEQUENCE	= 0;
	$TXFORWARD	= 1;
}

sub vtxValidateRx
{
	my ($data, $size) = @_;

	if ($size < $BUFFER_MIN || $size > $MAX)
	{
		return ($ERROR_INVALID_BUFFER_SIZE);
	}

	my $byte0 = lgo_BufferGet($data, 0);
	my $byte1 = lgo_BufferGet($data, 1);

	my $rxSequence = ($byte0 & $SEQUENCE_BITS) >> 1;

	if ($rxSequence != $RXSEQUENCE)
	{
		$RXSEQUENCE = ($rxSequence + 1) % $MODULUS;

		return ($ERROR_SEQUENCE_NUMBER_DISCREPANCY);
	}
	else
	{
		$RXSEQUENCE = ($RXSEQUENCE + 1) % $MODULUS;
	}

	my $rxSize = ($byte1 << 4) + (($byte0 & $SIZE_LOW_BITS) >> 4) + 1;

	if ($rxSize != $size)
	{
		return ($ERROR_SIZE_DISCREPANCY);
	}

	my $pattern = $byte0 & 1 ? $REVERSE : $FORWARD;
	my $i;

	for ($i = 2; $i < $size; $i++)
	{
		if (lgo_BufferGet($data, $i) != lgo_BufferGet($pattern, $i))
		{
			return ($ERROR_DATA_DISCREPANCY);
		}
	}

	return $ERROR_NONE;
}

1;

#******************************************************************************
#                                   End of File                               *
#******************************************************************************
